package jeliot.lang;

import java.lang.reflect.*;
import jeliot.parser.*;
import jeliot.theatre.*;
import jeliot.lang.*;

/**
  * @author Pekka Uronen
  *
  * created 25.8.1999
  */
public class ForeignMethodPointer extends MethodPointer {

    /** The method represented by this pointer.
      */
    private Method myMethod;
    
    /** The return type of this method.
      */
    private Type myType;

    public ForeignMethodPointer(Method meth) {
        this.myMethod = meth;
        this.myType = Type.getTypeByClass(meth.getReturnType());
    }

    public Type getReturnType() {
        return myType;
    }

    public boolean isStatic() {
        return Modifier.isStatic(myMethod.getModifiers());
    }

    public boolean isPrivate() {
        return Modifier.isPrivate(myMethod.getModifiers());
    }

    public Method getMethod() {
        return myMethod;
    }
    
    public String getName() {
        return myMethod.getName();
    }
    
    public ReferenceType getDeclaringClass() {
        return ForeignType.getType(myMethod.getDeclaringClass());
    }
    
    /** default invocation code */
    public Return invoke(Instance ins, Value[] args,
            PMethodInvocation invo, Director dir) {

        if (isStatic()) {
            // static invocation
            return dir.animateSFMInvocation(this, args);
        }
        return null;
    }
    
    public Value doInvoke(Instance ins, Value[] args) {    
        try {
            Object o = myMethod.invoke(null, unwrap(args));
            return new Value(myType, o);
        }
        catch (IllegalAccessException e) {
            e.printStackTrace(); 
        }
        catch (InvocationTargetException e) {
            e.printStackTrace(); 
        }
        return null;
    }
    
    public Value invokeStatic(Value[] args) {
        return doInvoke(null, args);
    }
    
    public Animator getAnimator(Value[] params) {
        String className = 
                "jeliot.script." +
                myMethod.getDeclaringClass().getName();
        String methodName = myMethod.getName();
        
        Class[] paramTypes = myMethod.getParameterTypes();
        try {
            Class anicla = Class.forName(className);
            Method anime = anicla.getMethod(methodName, paramTypes);    
            return (Animator)anime.invoke(null, unwrap(params));
        }
        catch (ClassNotFoundException e) {
            return new DefaultAnimator(this, params);    
        }
        catch (NoSuchMethodException e) {
            return new DefaultAnimator(this, params); 
        }
        catch (IllegalAccessException e) {
            return new DefaultAnimator(this, params); 
        }
        catch (InvocationTargetException e) {
            return new DefaultAnimator(this, params); 
        }
    }
    
    private Object[] unwrap(Value[] vals) {
        int n = (vals == null) ? 0 : vals.length;
        Object[] o = new Object[n];
        for (int i = 0; i < n; ++i) {
            o[i] = vals[i].getValue();
        }
        return o;
    }
    
}
