package jeliot.lang;

import java.util.*;
import jeliot.parser.*;
import jeliot.theatre.*;
import jeliot.FeatureNotImplementedException;

/**
  * MethodFrame is an instance of a method under execution. A method
  * frame is created runtime each time a method is called.
  *
  * @author Pekka Uronen
  *
  * created         9.8.1999
  */
public class MethodFrame {

    private PMethod method;
    private Stage stage;

    private int depth;
    
    private static class VariableAssociation {
        Variable var;
        PVariableDeclarator decl;
        
        VariableAssociation(Variable var, PVariableDeclarator decl) {
            this.var = var;
            this.decl = decl;
        }
    }
    private VariableAssociation[] vars;
    private int vcount;
    
    public MethodFrame(PMethod method) {
        this.method = method;
        vars = new VariableAssociation[method.getMaximumCount()];
    }
    
    public PMethod getMethod() {
        return method;
    }
    
    public Variable declareVariable(PVariableDeclarator decl) {
        Variable v = new Variable(decl);
        vars[vcount++] = new VariableAssociation(v, decl);
        return v;
    }

    public Variable getVariable(PVariableDeclarator decl) {
        for (int i = 0; i < vcount; ++i) {
            if (vars[i].decl == decl) {
                return vars[i].var;
            }
        }
        throw new RuntimeException("No Variable "+ decl);
    }

    public String getMethodName() {
        return method.getName();
    }

    public int getMaxVarCount() {
        return method.getMaximumCount();
    }

    public Stage getStage() {
        return stage;
    }

    public void setStage(Stage stage) {
        this.stage = stage;
    }

}
