package jeliot.parser;

import jeliot.lang.*;
import jeliot.theatre.*;
import jeliot.parser.error.*;

/**
  * PCastExpression represents a cast.
  *
  * @author Pekka Uronen
  *
  * created 23.8.1999
  */
public class PCastExpression extends PExpression {

    private PExpression expression;
    private PType ptype;

    public PCastExpression(PType ptype, Object dims, 
            PExpression expression) {
        
        this.expression = expression;
        this.ptype = ptype;

        expression.setParent(this);
    }

    public void acceptVisitor(ProgramTreeVisitor visitor) {
        
        visitor.enterCastExpression(this);
        expression.acceptVisitor(visitor);
        visitor.leaveCastExpression(this);
    }
    
    public PExpression getArgument() {
        return expression;
    }

    public Value evaluate(Director director) {
        super.evaluate(director);
        Value val = expression.evaluate(director);
        Value newValue = val.cast(getType());
        director.animateCastExpression(val, newValue);
        return newValue;
    }

    public void typeCheck(
            NameSpace space, 
            LocalVariableBundle locals, 
            PClass context) {
            
        Type t = ptype.resolveType(space, context);
        Type at = expression.getType();
        if (!at.mayBeCastTo(t)) {
            throw new IllegalCastException(this);
        }
        setType(t);
    }
    
}
