package jeliot.parser;

import java.util.*;
import jeliot.lang.*;
import jeliot.theatre.*;

/**
  * PIfThenElse represents an if-then or an if-then-else statement.
  *
  * @author Pekka Uronen
  *
  * created         6.8.1999
  * last modified   17.8.1999
  */
public class PIfThenElse extends PStatement {

    private PStatement thenSt;
    private PStatement elseSt;
    private PExpression check;
    
    protected PIfThenElse(PExpression check, PStatement thenSt, 
		    PStatement elseSt, int left, int right) {

		super(left, right);
		this.thenSt = thenSt;
		this.elseSt = elseSt;
		this.check = check;

		check.setParent(this);
		thenSt.setParent(this);
		if (elseSt != null) {
		    elseSt.setParent(this);
		}
    }

    public void acceptVisitor(ProgramTreeVisitor visitor) {
        visitor.visitIf(this);
        check.acceptVisitor(visitor);
        visitor.visitThen(this);
        thenSt.acceptVisitor(visitor);
        if (elseSt != null) {
            visitor.visitElse(this);
            elseSt.acceptVisitor(visitor);
        }
    }

    public PExpression getCheck() {
        return check;
    }

    void execute(Director director) throws Return {
        director.atStatement(this);
        Scratch scratch = director.openScratch();
        Value v = check.evaluate(director);
        Boolean b = (Boolean)v.getValue();
        director.atStatement(this);
        if (b.booleanValue()) {
            director.branchThen(this, v);
            director.closeScratch(scratch);
            thenSt.execute(director);
        }
        else if (elseSt != null) {
            director.branchElse(this, v);
            director.closeScratch(scratch);
            elseSt.execute(director);
        }
        else {
            director.skipIf(this, v);
        }
    }
}
