package jeliot.parser;

import jeliot.lang.*;
import jeliot.FeatureNotImplementedException;

/**
  * PModifiers class encapsulates a java type, either a primitive type
  * or a reference type.
  *
  * @author Pekka Uronen
  *
  * created         5.8.1999
  */
public class PType implements Cloneable {

    private int dimension = 0;
    private PName className;    // may be an interface name also

    private Type type;

    /** Start position of this type. */
    protected int left = Integer.MIN_VALUE;


    /** Creates a new primitive type. */
    public PType(Type type, int left) {
        this.type = type;
        this.left = left;
    }

    /** Creates a new reference type */
    public PType(PName className) {
        this.className = className;
        this.left = className.getLeftPos();

        className.classify(PName.TYPE);
    }

    /** Creates a new reference array type with specified dimensions.
      */
    public PType(PName className, int dimension) {
        this.className = className;
        this.dimension = dimension;
    }

    /** */
    Type resolveType(NameSpace space, PClass context) {
        if (this.type == null) {
            this.type = resolve(space, context, className);
        }
        if (dimension > 0) {
            // Array
            this.type = ArrayType.getType(type, dimension);
        }
        return this.type;
    }

    private static Type resolve(NameSpace space, PClass context,
            PName className) {

        ReferenceType p = space.findClass(className.toString());
        if (p == null) {
            throw new SemanticException("No class found:" + className);
        }
        return p;
    }

    Type getType() {
        return type;
    }

    /** Increases the dimension of this type by one, making this type
      * an array type. */
    void incDimension() {
        ++dimension;
    }

    /** Increases the dimension of this type by n, making this type
      * an array type. */
    void incDimension(int n) {
        dimension += n;
    }


    /** Sets the starting position of this type. Used when constructing
      * primitive types. */
    public void setLeftPos(int left) {
        this.left = left;
    }

    public int getLeftPos() {
        return left;
    }

    public String toString() {
        return type.toString();
    }


}
